import 'package:test/test.dart';
import 'package:flinq/flinq.dart';

final Iterable<int> _emptyCollection = <int>[];
final Iterable<int> _oneItemCollection = <int>[7];
final Iterable<int> _manyCollection = <int>[4, 3, 6, 1, 9, 5];

void iterableComparableTests() {
  minTests();
  maxTests();
  minOrNullTests();
  maxOrNullTests();
}

void minTests() {
  test(
    'Iterable<T extends Comparable>.min on empty collection',
    () =>
        expect(() => _emptyCollection.min, throwsA(TypeMatcher<StateError>())),
  );
  test(
    'Iterable<T extends Comparable>.min on collection with 1 element',
    () => expect(_oneItemCollection.min, 7),
  );
  test(
    'Iterable<T extends Comparable>.min on collection with many elements',
    () => expect(_manyCollection.min, 1),
  );
}

void maxTests() {
  test(
    'Iterable<T extends Comparable>.max on empty collection',
    () =>
        expect(() => _emptyCollection.max, throwsA(TypeMatcher<StateError>())),
  );
  test(
    'Iterable<T extends Comparable>.max on collection with 1 element',
    () => expect(_oneItemCollection.max, 7),
  );
  test(
    'Iterable<T extends Comparable>.max on collection with many elements',
    () => expect(_manyCollection.max, 9),
  );
}

void minOrNullTests() {
  test(
    'Iterable<T extends Comparable>.minOrNull on empty collection',
    () => expect(_emptyCollection.minOrNull, null),
  );
  test(
    'Iterable<T extends Comparable>.minOrNull on collection with 1 element',
    () => expect(_oneItemCollection.minOrNull, 7),
  );
  test(
    'Iterable<T extends Comparable>.minOrNull on collection with many elements',
    () => expect(_manyCollection.minOrNull, 1),
  );
}

void maxOrNullTests() {
  test(
    'Iterable<T extends Comparable>.maxOrNull on empty collection',
    () => expect(_emptyCollection.maxOrNull, null),
  );
  test(
    'Iterable<T extends Comparable>.maxOrNull on collection with 1 element',
    () => expect(_oneItemCollection.maxOrNull, 7),
  );
  test(
    'Iterable<T extends Comparable>.maxOrNull on collection with many elements',
    () => expect(_manyCollection.maxOrNull, 9),
  );
}
